@@ -58,6 +58,11 @@ static void gnome_print_config_finalize (GObject *object);
 
 static GObjectClass *parent_class = NULL;
 
+enum {
+	PROP_0,
+	PROP_NODE
+};
+
 GType
 gnome_print_config_get_type (void)
 {
@@ -77,16 +82,54 @@ gnome_print_config_get_type (void)
 	return type;
 }
 
+static void
+gnome_print_config_get_property (GObject *object, guint n, GValue *v,
+		GParamSpec *pspec)
+{
+	GnomePrintConfig *config = GNOME_PRINT_CONFIG (object);
+
+	switch (n) {
+	case PROP_NODE:
+		g_value_set_object (v, config->node);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, n, pspec);
+	}
+}
+
+static void
+gnome_print_config_set_property (GObject *object, guint n, const GValue *v,
+		GParamSpec *pspec)
+{
+	GnomePrintConfig *config = GNOME_PRINT_CONFIG (object);
+
+	switch (n) {
+	case PROP_NODE:
+		if (config->node)
+			g_object_unref (G_OBJECT (config->node));
+		config->node = g_value_get_object (v);
+		if (config->node)
+			g_object_ref (G_OBJECT (config->node));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, n, pspec);
+	}
+}
+
 static void
 gnome_print_config_class_init (GnomePrintConfigClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = (GObjectClass*) klass;
-
 	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = gnome_print_config_finalize;
+	object_class = (GObjectClass*) klass;
+	object_class->finalize     = gnome_print_config_finalize;
+	object_class->set_property = gnome_print_config_set_property;
+	object_class->get_property = gnome_print_config_get_property;
+	g_object_class_install_property (object_class, PROP_NODE,
+			g_param_spec_object ("node", "Node", "Node",
+				GPA_TYPE_NODE, G_PARAM_READWRITE));
 }
 
 static void
@@ -874,7 +917,3 @@ gnome_print_config_get_layout_data (GnomePrintConfig *config,
 
 	return lyd;
 }
-
-
-
-
