@@ -203,6 +203,8 @@ gboolean gnome_print_config_get_option (GnomePrintConfig *config, const guchar *
 #define GNOME_PRINT_KEY_LAYOUT_WIDTH  "Settings.Document.Page.Layout.Width"  /* Double value */
 #define GNOME_PRINT_KEY_LAYOUT_HEIGHT "Settings.Document.Page.Layout.Height" /* Double value */
 
+#define GNOME_PRINT_KEY_PAPER_SOURCE  "Settings.Output.PaperSource"          /* String value, like "Tray 1" */
+
 /* Master resolution, i.e. ink dots for color printer RGB resolution is usually smaller */
 #define GNOME_PRINT_KEY_RESOLUTION       "Settings.Output.Resolution"       /* String value, like 300x300 or 300dpi */
 #define GNOME_PRINT_KEY_RESOLUTION_DPI   "Settings.Output.Resolution.DPI"   /* Numeric value, like 300, if meaningful */
@@ -211,7 +213,13 @@ gboolean gnome_print_config_get_option (GnomePrintConfig *config, const guchar *
 
 /* These belong to 'Output' because PGL may implement multiple copies itself */
 #define GNOME_PRINT_KEY_NUM_COPIES "Settings.Output.Job.NumCopies" /* Number of copies */
+#define GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW "Settings.Output.Job.NonCollatedCopiesHW"
+#define GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW "Settings.Output.Job.CollatedCopiesHW"
+
 #define GNOME_PRINT_KEY_COLLATE    "Settings.Output.Job.Collate"   /* Boolean (true|yes|1 false|no|0) */
+#define GNOME_PRINT_KEY_DUPLEX    "Settings.Output.Job.Duplex"   /* Boolean (true|yes|1 false|no|0) */
+#define GNOME_PRINT_KEY_TUMBLE    "Settings.Output.Job.Tumble"   /* Boolean (true|yes|1 false|no|0) */
+#define GNOME_PRINT_KEY_HOLD    "Settings.Output.Job.Hold"   /* String value, like no-hold|indefinite|day-time|evening|night|weekend|second-shift|third-shift*/
 
 /* These are ignored by libgnomeprint, but you may want to get/set/inspect these */
 /* Libgnomeprintui uses these for displaying margin symbols */
