@@ -365,8 +365,6 @@ gnome_print_encode_drow (const guchar *in, guchar *out, gint in_size, guchar *se
 			{
 				if (out[p1]==31)
 				{
-					/* Shakira really rules, so does the -- operator
-					 that's why we use it in the next line */ 
 					changed_bytes--;
 					/* Since we used more that 1 ctrl byte we know
 					   that the 1st ctrl byte has the 5 lower bits
@@ -434,7 +432,7 @@ gnome_print_encode_drow (const guchar *in, guchar *out, gint in_size, guchar *se
  * returns the bytes that the apps should allocate
  * for the "out" buffer of a hex encodede stream
  * 
- * Return Value: the size of a ascii85 buffer in a WCS 
+ * Return Value: the size of a hex buffer in a WCS 
  **/
 gint
 gnome_print_encode_hex_wcs (gint size)
@@ -482,6 +480,9 @@ gnome_print_encode_hex (const guchar *in, guchar *out, gint in_size)
 		}
 	}
 
+	if (col != 0)
+		out [p1++] = '\n';
+	
 	return p1;
 }
 
@@ -508,12 +509,16 @@ gnome_print_decode_hex_wcs (gint size)
 static inline gint
 hex_2_dec (guchar upper, guchar lower)
 {
-	if (upper > '9')
+	if (upper > '9') {
+		upper |= 0x20;
 		upper -= 39;
-	if (lower > '9')
+	}
+	if (lower > '9') {
+		lower |= 0x20;
 		lower -= 39;
+	}
 	       
-	return ((upper - '0')*16) + lower - '0';
+	return ((upper - '0') << 4) + lower - '0';
 }
 
 /**
