@@ -29,7 +29,6 @@
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <locale.h>
 
 #include <libart_lgpl/art_affine.h>
@@ -37,14 +36,29 @@
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-meta.h>
 #include <libgnomeprint/gnome-print-transport.h>
-#include <libgnomeprint/gnome-print-multipage.h>
-#include <libgnomeprint/gnome-print-job-private.h>
+#include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprint/gnome-print-config-private.h>
 #include <libgnomeprint/gnome-print-i18n.h>
 
-typedef struct _JOBPrivate JOBPrivate;
+#include <libgnomeprint/gnome-print-filter.h>
 
-struct _JOBPrivate {
+typedef struct _GnomePrintJobPrivate GnomePrintJobPrivate;
+
+struct _GnomePrintJob {
+	GObject object;
+
+	GnomePrintConfig *config;
+	GnomePrintContext *meta;
+	guchar *input_file;
+
+	GnomePrintJobPrivate *priv;
+};
+
+struct _GnomePrintJobClass {
+	GObjectClass parent_class;
+};
+
+struct _GnomePrintJobPrivate {
 	/* closed flag */
 	guint closed : 1;
 
@@ -68,18 +82,19 @@ struct _JOBPrivate {
 
 enum
 {
-  PROP_0,
-  PROP_CONFIG
+	PROP_0,
+	PROP_CONFIG,
+	PROP_CONTEXT
 };
  
 
-#define GNOME_PRINT_JOB_CLOSED(m) (((JOBPrivate *) (m)->priv)->closed)
+#define GNOME_PRINT_JOB_CLOSED(m) ((m)->priv->closed)
 
 static void gnome_print_job_class_init (GnomePrintJobClass *klass);
 static void gnome_print_job_init (GnomePrintJob *job);
 static void gnome_print_job_finalize (GObject *object);
 
-static void job_update_layout_data (GnomePrintJob *job);
+static void gnome_print_job_update_layout_data (GnomePrintJob *job);
 static void job_parse_config_data (GnomePrintJob *job);
 static void job_clear_config_data (GnomePrintJob *job);
 
@@ -121,20 +136,21 @@ gnome_print_job_class_init (GnomePrintJobClass *klass)
 	
 	object_class = (GObjectClass *) klass;
 
+	object_class->finalize     = gnome_print_job_finalize;
 	object_class->set_property = gnome_print_job_set_property;
 	object_class->get_property = gnome_print_job_get_property;
 
-	g_object_class_install_property (object_class,
-                                   PROP_CONFIG,
-                                   g_param_spec_object ("config",
-                                                        _("Job Configuration"),
-                                                        _("The configuration for the print job"),
-                                                        GNOME_TYPE_PRINT_CONFIG,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  
-	parent_class = g_type_class_peek_parent (klass);
+	g_object_class_install_property (object_class, PROP_CONFIG,
+		g_param_spec_object ("config", _("Job Configuration"),
+			_("The configuration for the print job"),
+			GNOME_TYPE_PRINT_CONFIG,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class, PROP_CONTEXT,
+		g_param_spec_object ("context", _("Context"),
+			_("The context for the print job"),
+			GNOME_TYPE_PRINT_META, G_PARAM_READWRITE));
 
-	object_class->finalize = gnome_print_job_finalize;
+	parent_class = g_type_class_peek_parent (klass);
 }
 
 static void
@@ -142,11 +158,11 @@ gnome_print_job_init (GnomePrintJob *job)
 {
 	job->config = NULL;
 
-	job->meta = NULL;
+	job->meta = g_object_new (GNOME_TYPE_PRINT_META, NULL);
 
 	job->input_file = NULL;
 
-	job->priv = g_new0 (JOBPrivate, 1);
+	job->priv = g_new0 (GnomePrintJobPrivate, 1);
 	job_clear_config_data (job);
 }
 
@@ -183,48 +199,44 @@ static void
 gnome_print_job_get_property (GObject *object, guint prop_id, GValue *value,
 	                     GParamSpec *pspec)
 {
-  GnomePrintJob *job = GNOME_PRINT_JOB (object);
+	GnomePrintJob *job = GNOME_PRINT_JOB (object);
 
-  switch (prop_id)
-    {
-    case PROP_CONFIG:
-      g_value_set_object (value, gnome_print_job_get_config (job));
-      break;
-  
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
+	switch (prop_id) {
+	case PROP_CONFIG:
+		g_value_set_object (value, job->config);
+		break;
+	case PROP_CONTEXT:
+		g_value_set_object (value, job->meta);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 static void
 gnome_print_job_set_property (GObject * object, guint prop_id,
 	                     const GValue * value, GParamSpec * pspec)
 {
-  GnomePrintJob *job = GNOME_PRINT_JOB (object);
-  GnomePrintConfig *config = NULL;
+	GnomePrintJob *job = GNOME_PRINT_JOB (object);
 
-  switch (prop_id)
-    {
-    case PROP_CONFIG:
-	    config = GNOME_PRINT_CONFIG (g_value_get_object (value));
-	    if (job->config) {
-		    gnome_print_config_unref (job->config);
-		    job->config = NULL;
-	    }
-	    
-	    if(config) {
-		    gnome_print_config_ref (config);
-		    job->config = config;
-	    } else {
-		    /* Use the default: */
-		    job->config = gnome_print_config_default ();
-	    }
-	    break;
-    default:
-	    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-	    break;
-    }
+	switch (prop_id) {
+	case PROP_CONFIG:
+		if (job->config) gnome_print_config_unref (job->config);
+		job->config = g_value_get_object (value);
+		if (job->config) gnome_print_config_ref (job->config);
+		else job->config = gnome_print_config_default ();
+		break;
+	case PROP_CONTEXT:
+		if (job->meta) g_object_unref (job->meta);
+		job->meta = g_value_get_object (value);
+		if (job->meta) g_object_ref (G_OBJECT (job->meta));
+		else job->meta = g_object_new (GNOME_TYPE_PRINT_META, NULL);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 
@@ -285,9 +297,6 @@ gnome_print_job_get_context (GnomePrintJob *job)
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), NULL);
 	g_return_val_if_fail (job->input_file == NULL, NULL);
 
-	if (job->meta == NULL)
-		job->meta = gnome_print_meta_new ();
-
 	g_object_ref (G_OBJECT (job->meta));
 
 	return job->meta;
@@ -311,9 +320,77 @@ gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
 	g_return_if_fail (GNOME_IS_PRINT_JOB (job));
 	g_return_if_fail (input_file != NULL);
 	g_return_if_fail (job->input_file == NULL);
-	g_return_if_fail (job->meta == NULL);
 
-	job->input_file = g_strdup (input_file);
+	job->input_file = (guchar *) g_strdup (input_file);
+}
+
+static void
+gnome_print_job_setup_context (GnomePrintJob *job, GnomePrintContext *pc)
+{
+	GnomePrintFilter *filter = NULL;
+	gchar *description;
+
+	g_return_if_fail (GNOME_IS_PRINT_JOB (job));
+	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (pc));
+
+	g_object_set (G_OBJECT (pc), "filter", NULL, NULL);
+
+	/* Explicit filter description */
+	description = (gchar *) gnome_print_config_get (job->config,
+			(const guchar *) "Settings.Output.Job.Filter");
+	if (description) {
+		GError *e = NULL;
+
+		filter = gnome_print_filter_new_from_description (description, &e);
+		if (e) {
+			g_warning ("Problem while creating filter from '%s': %s",
+				description, e->message);
+			g_error_free (e);
+		}
+		g_free (description);
+		if (filter) {
+			g_object_set (G_OBJECT (pc), "filter", filter, NULL);
+			g_object_unref (G_OBJECT (filter));
+		}
+	}
+
+	gnome_print_job_update_layout_data (job);
+	if (job->priv->LY_LIST && g_list_length (job->priv->LY_LIST) && 
+			!((g_list_length (job->priv->LY_LIST) == 1) &&
+				(((gdouble *) g_list_nth_data (job->priv->LY_LIST, 0)) [0] == 1.) &&
+				(((gdouble *) g_list_nth_data (job->priv->LY_LIST, 0)) [1] == 0.) &&
+				(((gdouble *) g_list_nth_data (job->priv->LY_LIST, 0)) [2] == 0.) &&
+				(((gdouble *) g_list_nth_data (job->priv->LY_LIST, 0)) [3] == 1.) &&
+				(((gdouble *) g_list_nth_data (job->priv->LY_LIST, 0)) [4] == 0.) &&
+				(((gdouble *) g_list_nth_data (job->priv->LY_LIST, 0)) [5] == 0.))) {
+		GnomePrintFilter *multipage;
+		GValue vd = {0,};
+		GValueArray *va = g_value_array_new (0);
+		guint i, npages;
+		
+		npages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
+		npages = (npages + job->priv->num_affines - 1) / job->priv->num_affines;
+		g_value_init (&vd, G_TYPE_DOUBLE);
+		for (i = g_list_length (job->priv->LY_LIST); i > 0; i--) {
+			gdouble *a = g_list_nth_data (job->priv->LY_LIST, i - 1);
+			guint j;
+
+			for (j = 6; j > 0; j--) {
+				g_value_set_double (&vd, a[j - 1]);
+				g_value_array_prepend (va, &vd);
+			}
+		}
+		g_value_unset (&vd);
+		multipage = gnome_print_filter_new_from_module_name ("multipage",
+				"affines", va, NULL);
+		g_value_array_free (va);
+		if (multipage) {
+			if (filter)
+				gnome_print_filter_append_predecessor (filter, multipage);
+			g_object_set (G_OBJECT (pc), "filter", multipage, NULL);
+			g_object_unref (G_OBJECT (multipage));
+		}
+	}
 }
 
 /**
@@ -330,24 +407,24 @@ gnome_print_job_set_file (GnomePrintJob *job, gchar *input_file)
 int
 gnome_print_job_get_pages (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
-	gint mp;
+	gint n;
+	GnomePrintContext *pc;
 
-	g_return_val_if_fail (job != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), 0);
-	g_return_val_if_fail (job->meta, 0);
 	g_return_val_if_fail (GNOME_PRINT_JOB_CLOSED (job), 0);
 
-	job_update_layout_data (job);
+	/*
+	 * Filters can suppress or add pages. We therefore just pass everything
+	 * through the filter pipeline and count the number of pages that come 
+	 * out of it.
+	 */
+	pc = g_object_new (GNOME_TYPE_PRINT_META, NULL);
+	gnome_print_job_setup_context (job, pc);
+	gnome_print_meta_render (GNOME_PRINT_META (job->meta), pc);
+	n = gnome_print_meta_get_pages (GNOME_PRINT_META (pc));
+	g_object_unref (G_OBJECT (pc));
 
-	mp = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
-
-	pp = job->priv;
-
-	if (pp->num_affines <= 1)
-		return mp;
-
-	return (mp + pp->num_affines - 1) / pp->num_affines;
+	return n;
 }
 
 /**
@@ -380,27 +457,23 @@ gnome_print_job_get_page_size_from_config (GnomePrintConfig *config, gdouble *wi
 gboolean
 gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *height)
 {
-	JOBPrivate *pp;
-
 	g_return_val_if_fail (job != NULL, FALSE);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), FALSE);
 	g_return_val_if_fail (width != NULL, FALSE);
 	g_return_val_if_fail (height != NULL, FALSE);
 
-	pp = job->priv;
+	gnome_print_job_update_layout_data (job);
 
-	job_update_layout_data (job);
-
-	if (pp->LY_LIST) {
+	if (job->priv->LY_LIST) {
 		if (width)
-			*width = pp->LW;
+			*width = job->priv->LW;
 		if (height)
-			*height = pp->LH;
+			*height = job->priv->LH;
 	} else {
 		if (width)
-			*width = pp->pw;
+			*width = job->priv->pw;
 		if (height)
-			*height = pp->ph;
+			*height = job->priv->ph;
 	}
 
 	return TRUE;
@@ -419,25 +492,13 @@ gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *heig
 gint
 gnome_print_job_close (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
-
 	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (job->input_file == NULL, 0);
+	g_return_val_if_fail (!GNOME_PRINT_JOB_CLOSED (job), GNOME_PRINT_ERROR_UNKNOWN);
 
-	if (!job->meta)
-		job->meta = gnome_print_meta_new ();
-
-	pp = job->priv;
-
-	if (!pp->closed) {
-		pp->closed = TRUE;
-		return gnome_print_context_close (job->meta);
-	}
-
-	g_warning ("gnome_print_job_close can only be called once\n");
-
-	return GNOME_PRINT_ERROR_UNKNOWN;
+	job->priv->closed = TRUE;
+	return gnome_print_context_close (job->meta);
 }
 
 /**
@@ -462,7 +523,7 @@ gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
 	const guchar *buf;
 	gint blen;
 
-	drivername = gnome_print_config_get (job->config, "Settings.Engine.Backend.Driver");
+	drivername = (gchar *) gnome_print_config_get (job->config, (const guchar *) "Settings.Engine.Backend.Driver");
 
 	if (drivername == NULL)
 		return FALSE;
@@ -474,7 +535,7 @@ gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
 
 	*retval = GNOME_PRINT_ERROR_UNKNOWN;
 	
-	gnome_print_config_get_boolean (job->config, "Settings.Output.Job.PrintToFile", &print_to_file);
+	gnome_print_config_get_boolean (job->config, (const guchar *) "Settings.Output.Job.PrintToFile", &print_to_file);
 	if (!print_to_file) {
 		g_warning ("Metadata printer should always be print to file");
 		goto metadata_printer_done;
@@ -491,7 +552,7 @@ gnome_print_job_metadata_printer (GnomePrintJob *job, gint *retval)
 
 	gnome_print_transport_open  (transport);
 	gnome_print_transport_write (transport, buf, blen);
-	gnome_print_transport_write (transport, "GNOME_METAFILE_END", strlen ("GNOME_METAFILE_END"));
+	gnome_print_transport_write (transport, (const guchar *) "GNOME_METAFILE_END", strlen ("GNOME_METAFILE_END"));
 	gnome_print_transport_close (transport);
 	
 	*retval = GNOME_PRINT_OK;
@@ -515,13 +576,11 @@ metadata_printer_done:
 gint
 gnome_print_job_print (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	GnomePrintContext *ctx;
+	GnomePrintFilter *filter = NULL;
 	gint lpages, copies, nstacks, npages, nsheets;
-	gboolean collate, is_multipage, hwcopies;
+	gboolean collate, hwcopies;
 	gint stack;
-	const guchar *buf;
-	gint blen;
 	gint ret;
 
 	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
@@ -533,9 +592,7 @@ gnome_print_job_print (GnomePrintJob *job)
 		return gnome_print_transport_print_file (transport, job->input_file);
 	}
 
-	pp = job->priv;
-
-	if (!pp->closed) {
+	if (!job->priv->closed) {
 		g_warning ("You should call gnome_print_job_close before calling\n"
 			   "gnome_print_job_print\n");
 		gnome_print_job_close (job);
@@ -545,39 +602,25 @@ gnome_print_job_print (GnomePrintJob *job)
 	lpages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
 	if (lpages < 1)
 		return GNOME_PRINT_OK;
-	npages = lpages;
 
 	/* Special case where printer is a Metadata print to file */
 	if (gnome_print_job_metadata_printer (job, &ret))
 		return ret;
 
-	/* Create the "real" context */
+	/* Create the "real" context and set it up. */
 	ctx = gnome_print_context_new (job->config);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-
-	/* Update the layout data */
-	is_multipage = FALSE;
-	job_update_layout_data (job);
-	if (pp->LY_LIST) {
-		GnomePrintContext *mp;
-		/* Find out physical page count */
-		npages = (npages + pp->num_affines - 1) / pp->num_affines;
-		/* Create multipage context */
-		mp = gnome_print_multipage_new (ctx, pp->LY_LIST);
-		g_object_unref (G_OBJECT (ctx));
-		ctx = mp;
-		is_multipage = TRUE;
-	}
+	gnome_print_job_setup_context (job, ctx);
+	g_object_get (G_OBJECT (ctx), "filter", &filter, NULL);
 
 	collate = FALSE;
-	gnome_print_config_get_boolean (job->config, GNOME_PRINT_KEY_COLLATE, &collate);
+	gnome_print_config_get_boolean (job->config, (const guchar *) GNOME_PRINT_KEY_COLLATE, &collate);
 	copies = 1;
-	gnome_print_config_get_int (job->config, GNOME_PRINT_KEY_NUM_COPIES, &copies);
+	gnome_print_config_get_int (job->config, (const guchar *) GNOME_PRINT_KEY_NUM_COPIES, &copies);
 	hwcopies = FALSE;
 	gnome_print_config_get_boolean (job->config, 
 					collate 
-					? GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW
-					: GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW, 
+					? (const guchar *) GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW
+					: (const guchar *) GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW, 
 					&hwcopies);
 
 	if (hwcopies)
@@ -590,10 +633,8 @@ gnome_print_job_print (GnomePrintJob *job)
 		nstacks = 1;
 		nsheets = copies;
 	}
-	
-	buf  = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
-	blen = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
 
+	npages = (lpages + job->priv->num_affines - 1) / job->priv->num_affines;
 	for (stack = 0; stack < nstacks; stack++) {
 		gint page;
 		for (page = 0; page < npages; page++) {
@@ -601,15 +642,15 @@ gnome_print_job_print (GnomePrintJob *job)
 			for (sheet = 0; sheet < nsheets; sheet++) {
 				gint start, i;
 				/* Render physical page */
-				start = page * pp->num_affines;
-				for (i = start; (i < (start + pp->num_affines)) && (i < lpages); i++) {
-					ret = gnome_print_meta_render_data_page (ctx, buf, blen, i, TRUE);
+				start = page * job->priv->num_affines;
+				for (i = start; (i < (start + job->priv->num_affines)) && (i < lpages); i++) {
+					ret = gnome_print_meta_render_page (GNOME_PRINT_META (job->meta),
+							ctx, i, TRUE);
 					g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
 				}
 				/* Flush physical page */
-				if (is_multipage) {
-					gnome_print_multipage_finish_page (GNOME_PRINT_MULTIPAGE (ctx));
-				}
+				if (filter)
+					gnome_print_filter_flush (filter);
 			}
 		}
 		if (stack + 1 < nstacks) {
@@ -618,6 +659,9 @@ gnome_print_job_print (GnomePrintJob *job)
 		}
 	}
 
+	if (NULL != filter)
+	    g_object_unref (G_OBJECT (filter));
+
 	ret = gnome_print_context_close (ctx);
 	g_object_unref (G_OBJECT (ctx));
 
@@ -638,37 +682,10 @@ gnome_print_job_print (GnomePrintJob *job)
 gint
 gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 {
-	JOBPrivate *pp;
-	const guchar *data;
-	gint len;
-	gint ret;
-
-	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
-	pp = job->priv;
-
-	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	len = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (len > 0, GNOME_PRINT_ERROR_UNKNOWN);
-
-	job_update_layout_data (job);
-	if (pp->LY_LIST) {
-		GnomePrintContext *mp;
-		mp = gnome_print_multipage_new (ctx, pp->LY_LIST);
-		ret = gnome_print_meta_render_data (mp, data, len);
-		gnome_print_multipage_finish_page (GNOME_PRINT_MULTIPAGE (mp));
-		g_object_unref (G_OBJECT (mp));
-	} else {
-		ret = gnome_print_meta_render_data (ctx, data, len);
-	}
-
-	return ret;
+	gnome_print_job_setup_context (job, ctx);
+	return gnome_print_meta_render (GNOME_PRINT_META (job->meta), ctx);
 }
 
 /**
@@ -685,54 +702,16 @@ gnome_print_job_render (GnomePrintJob *job, GnomePrintContext *ctx)
 gint
 gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint page, gboolean pageops)
 {
-	JOBPrivate *pp;
-	const guchar *data;
-	gint len, pages;
-	gint ret;
+	GnomePrintContext *pc;
 
-	g_return_val_if_fail (job != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (ctx != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (ctx), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->priv, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (job->meta, GNOME_PRINT_ERROR_UNKNOWN);
 
-	pp = job->priv;
-
-	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	len = gnome_print_meta_get_length (GNOME_PRINT_META (job->meta));
-	g_return_val_if_fail (len > 0, GNOME_PRINT_ERROR_UNKNOWN);
-	pages = gnome_print_meta_get_pages (GNOME_PRINT_META (job->meta));
-
-	job_update_layout_data (job);
-	if (pp->LY_LIST) {
-		GnomePrintContext *meta, *mp;
-		gint start, p;
-
-		meta = gnome_print_meta_new ();
-		g_return_val_if_fail (meta != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-		/* Create wrapper multipage */
-		mp = gnome_print_multipage_new (meta, pp->LY_LIST);
-		start = page * pp->num_affines;
-		/* Render job pages */
-		for (p = start; (p < pages) && (p < start + pp->num_affines); p++) {
-			gnome_print_meta_render_data_page (mp, data, len, p, TRUE);
-		}
-		/* Finish multipage */
-		gnome_print_context_close (mp);
-		g_object_unref (G_OBJECT (mp));
-		/* Render page */
-		data = gnome_print_meta_get_buffer (GNOME_PRINT_META (meta));
-		len = gnome_print_meta_get_length (GNOME_PRINT_META (meta));
-		ret = gnome_print_meta_render_data_page (ctx, data, len, 0, pageops);
-		/* Release meta */
-		g_object_unref (G_OBJECT (meta));
-	} else {
-		ret = gnome_print_meta_render_data_page (ctx, data, len, page, pageops);
-	}
-
-	return ret;
+	pc = g_object_new (GNOME_TYPE_PRINT_META, NULL);
+	gnome_print_job_setup_context (job, pc);
+	gnome_print_meta_render (GNOME_PRINT_META (job->meta), pc);
+	gnome_print_meta_render_page (GNOME_PRINT_META (pc), ctx, page, pageops);
+	g_object_unref (G_OBJECT (pc));
+	return GNOME_PRINT_OK;
 }
 
 /* We need:
@@ -769,9 +748,8 @@ gnome_print_job_render_page (GnomePrintJob *job, GnomePrintContext *ctx, gint pa
 #endif
 
 static void
-job_update_layout_data (GnomePrintJob *job)
+gnome_print_job_update_layout_data (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	ArtDRect area, r;
 	gdouble t;
 	gdouble a[6];
@@ -779,112 +757,107 @@ job_update_layout_data (GnomePrintJob *job)
 
 	g_return_if_fail (job->priv);
 
-	pp = job->priv;
-
 	job_parse_config_data (job);
 
 	/* Now comes the fun part */
 
-	if (pp->num_affines < 1)
+	if (job->priv->num_affines < 1)
 		return;
-	if ((fabs (pp->pw) < EPSILON) || (fabs (pp->ph) < EPSILON))
+	if ((fabs (job->priv->pw) < EPSILON) || (fabs (job->priv->ph) < EPSILON))
 		return;
 
 	/* Initial setup */
 	/* Calculate PP2PA */
 	/* We allow only rectilinear setups, so we can cheat */
-	pp->PP2PA[0] = pp->porient[0];
-	pp->PP2PA[1] = pp->porient[1];
-	pp->PP2PA[2] = pp->porient[2];
-	pp->PP2PA[3] = pp->porient[3];
-	t = pp->pw * pp->PP2PA[0] + pp->ph * pp->PP2PA[2];
-	pp->PP2PA[4] = (t < 0) ? -t : 0.0;
-	t = pp->pw * pp->PP2PA[1] + pp->ph * pp->PP2PA[3];
-	pp->PP2PA[5] = (t < 0) ? -t : 0.0;
-	PRINT_AFFINE ("PP2PA:", &pp->PP2PA[0]);
+	job->priv->PP2PA[0] = job->priv->porient[0];
+	job->priv->PP2PA[1] = job->priv->porient[1];
+	job->priv->PP2PA[2] = job->priv->porient[2];
+	job->priv->PP2PA[3] = job->priv->porient[3];
+	t = job->priv->pw * job->priv->PP2PA[0] + job->priv->ph * job->priv->PP2PA[2];
+	job->priv->PP2PA[4] = (t < 0) ? -t : 0.0;
+	t = job->priv->pw * job->priv->PP2PA[1] + job->priv->ph * job->priv->PP2PA[3];
+	job->priv->PP2PA[5] = (t < 0) ? -t : 0.0;
+	PRINT_AFFINE ("PP2PA:", &job->priv->PP2PA[0]);
 
 	/* PPDP - Physical Page Dimensions in Printer */
 	/* A: PhysicalPage X PhysicalOrientation X TRANSLATE -> Physical Page in Printer */
 	area.x0 = 0.0;
 	area.y0 = 0.0;
-	area.x1 = pp->pw;
-	area.y1 = pp->ph;
-	art_drect_affine_transform (&r, &area, pp->PP2PA);
-	pp->PAW = r.x1 - r.x0;
-	pp->PAH = r.y1 - r.y0;
-	if ((pp->PAW < EPSILON) || (pp->PAH < EPSILON))
+	area.x1 = job->priv->pw;
+	area.y1 = job->priv->ph;
+	art_drect_affine_transform (&r, &area, job->priv->PP2PA);
+	job->priv->PAW = r.x1 - r.x0;
+	job->priv->PAH = r.y1 - r.y0;
+	if ((job->priv->PAW < EPSILON) || (job->priv->PAH < EPSILON))
 		return;
 
 	/* Now we have to find the size of layout page */
 	/* Again, knowing that layouts are rectilinear helps us */
-	art_affine_invert (a, pp->affines);
+	art_affine_invert (a, job->priv->affines);
 	PRINT_AFFINE ("INV LY:", &a[0]);
-	pp->LYW = pp->lyw * fabs (pp->pw * a[0] + pp->ph * a[2]);
-	pp->LYH = pp->lyh * fabs (pp->pw * a[1] + pp->ph * a[3]);
-	PRINT_2 ("LY Dimensions:", pp->LYW, pp->LYH);
+	job->priv->LYW = job->priv->lyw * fabs (job->priv->pw * a[0] + job->priv->ph * a[2]);
+	job->priv->LYH = job->priv->lyh * fabs (job->priv->pw * a[1] + job->priv->ph * a[3]);
+	PRINT_2 ("LY Dimensions:", job->priv->LYW, job->priv->LYH);
 
 	/* Calculate LP2LY */
 	/* We allow only rectilinear setups, so we can cheat */
-	pp->LP2LY[0] = pp->lorient[0];
-	pp->LP2LY[1] = pp->lorient[1];
-	pp->LP2LY[2] = pp->lorient[2];
-	pp->LP2LY[3] = pp->lorient[3];
+	job->priv->LP2LY[0] = job->priv->lorient[0];
+	job->priv->LP2LY[1] = job->priv->lorient[1];
+	job->priv->LP2LY[2] = job->priv->lorient[2];
+	job->priv->LP2LY[3] = job->priv->lorient[3];
 	/* Delay */
-	pp->LP2LY[4] = 0.0;
-	pp->LP2LY[5] = 0.0;
+	job->priv->LP2LY[4] = 0.0;
+	job->priv->LP2LY[5] = 0.0;
 	/* Meanwhile find logical width and height */
 	area.x0 = 0.0;
 	area.y0 = 0.0;
-	area.x1 = pp->LYW;
-	area.y1 = pp->LYH;
-	art_affine_invert (a, pp->LP2LY);
+	area.x1 = job->priv->LYW;
+	area.y1 = job->priv->LYH;
+	art_affine_invert (a, job->priv->LP2LY);
 	art_drect_affine_transform (&r, &area, a);
-	pp->LW = r.x1 - r.x0;
-	pp->LH = r.y1 - r.y0;
-	if ((pp->LW < EPSILON) || (pp->LH < EPSILON))
+	job->priv->LW = r.x1 - r.x0;
+	job->priv->LH = r.y1 - r.y0;
+	if ((job->priv->LW < EPSILON) || (job->priv->LH < EPSILON))
 		return;
-	PRINT_2 ("L Dimensions", pp->LW, pp->LH);
+	PRINT_2 ("L Dimensions", job->priv->LW, job->priv->LH);
 	/* Now complete matrix calculation */
-	t = pp->LW * pp->LP2LY[0] + pp->LH * pp->LP2LY[2];
-	pp->LP2LY[4] = (t < 0) ? -t : 0.0;
-	t = pp->LW * pp->LP2LY[1] + pp->LH * pp->LP2LY[3];
-	pp->LP2LY[5] = (t < 0) ? -t : 0.0;
-	PRINT_AFFINE ("LP2LY:", &pp->LP2LY[0]);
+	t = job->priv->LW * job->priv->LP2LY[0] + job->priv->LH * job->priv->LP2LY[2];
+	job->priv->LP2LY[4] = (t < 0) ? -t : 0.0;
+	t = job->priv->LW * job->priv->LP2LY[1] + job->priv->LH * job->priv->LP2LY[3];
+	job->priv->LP2LY[5] = (t < 0) ? -t : 0.0;
+	PRINT_AFFINE ("LP2LY:", &job->priv->LP2LY[0]);
 
 	/* Good, now generate actual layout matrixes */
 
-	pp->LY_AFFINES = g_new (gdouble, 6 * pp->num_affines);
+	job->priv->LY_AFFINES = g_new (gdouble, 6 * job->priv->num_affines);
 
 	/* Extra fun */
 
-	for (i = 0; i < pp->num_affines; i++) {
+	for (i = 0; i < job->priv->num_affines; i++) {
 		gdouble ly2p[6];
 		gdouble *ly2pa;
 		/* Calculate Layout -> Physical Page affine */
-		memcpy (ly2p, pp->affines + 6 * i, 6 * sizeof (gdouble));
-		ly2p[4] *= pp->pw;
-		ly2p[5] *= pp->ph;
+		memcpy (ly2p, job->priv->affines + 6 * i, 6 * sizeof (gdouble));
+		ly2p[4] *= job->priv->pw;
+		ly2p[5] *= job->priv->ph;
 		/* PRINT_AFFINE ("Layout -> Physical:", &l2p[0]); */
-		art_affine_multiply (pp->LY_AFFINES + 6 * i, pp->LP2LY, ly2p);
+		art_affine_multiply (job->priv->LY_AFFINES + 6 * i, job->priv->LP2LY, ly2p);
 		ly2pa = g_new (gdouble, 6);
-		art_affine_multiply (ly2pa, pp->LY_AFFINES + 6 * i, pp->PP2PA);
-		pp->LY_LIST = g_list_prepend (pp->LY_LIST, ly2pa);
+		art_affine_multiply (ly2pa, job->priv->LY_AFFINES + 6 * i, job->priv->PP2PA);
+		job->priv->LY_LIST = g_list_prepend (job->priv->LY_LIST, ly2pa);
 	}
 
-	pp->LY_LIST = g_list_reverse (pp->LY_LIST);
+	job->priv->LY_LIST = g_list_reverse (job->priv->LY_LIST);
 }
 
 static void
 job_parse_config_data (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
 	const GnomePrintUnit *unit;
 	GPANode *layout;
 
 	g_return_if_fail (job->priv);
 
-	pp = job->priv;
-
 	job_clear_config_data (job);
 
 	g_return_if_fail (job->config);
@@ -893,28 +866,28 @@ job_parse_config_data (GnomePrintJob *job)
 
 
 	/* Physical size */
-	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_WIDTH, &pp->pw, &unit)) {
-		gnome_print_convert_distance (&pp->pw, unit, GNOME_PRINT_PS_UNIT);
+	if (gnome_print_config_get_length (job->config, (const guchar *) GNOME_PRINT_KEY_PAPER_WIDTH, &job->priv->pw, &unit)) {
+		gnome_print_convert_distance (&job->priv->pw, unit, GNOME_PRINT_PS_UNIT);
 	}
-	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &pp->ph, &unit)) {
-		gnome_print_convert_distance (&pp->ph, unit, GNOME_PRINT_PS_UNIT);
+	if (gnome_print_config_get_length (job->config, (const guchar *) GNOME_PRINT_KEY_PAPER_HEIGHT, &job->priv->ph, &unit)) {
+		gnome_print_convert_distance (&job->priv->ph, unit, GNOME_PRINT_PS_UNIT);
 	}
 	/* Physical orientation */
-	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, pp->porient);
+	gnome_print_config_get_transform (job->config, (const guchar *) GNOME_PRINT_KEY_PAPER_ORIENTATION_MATRIX, job->priv->porient);
 	/* Logical orientation */
-	gnome_print_config_get_transform (job->config, GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, pp->lorient);
+	gnome_print_config_get_transform (job->config, (const guchar *) GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, job->priv->lorient);
 	/* Layout size */
-	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_WIDTH, &pp->lyw);
-	gnome_print_config_get_double (job->config, GNOME_PRINT_KEY_LAYOUT_HEIGHT, &pp->lyh);
+	gnome_print_config_get_double (job->config, (const guchar *) GNOME_PRINT_KEY_LAYOUT_WIDTH, &job->priv->lyw);
+	gnome_print_config_get_double (job->config, (const guchar *) GNOME_PRINT_KEY_LAYOUT_HEIGHT, &job->priv->lyh);
 
 	/* Now come the affines */
-	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (job->config), GNOME_PRINT_KEY_LAYOUT);
+	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (job->config), (const guchar *) GNOME_PRINT_KEY_LAYOUT);
 	if (layout) {
 		gint numlp;
 		numlp = 0;
-		if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
+		if (gpa_node_get_int_path_value (layout, (const guchar *) "LogicalPages", &numlp) && (numlp > 0)) {
 			GPANode *pages;
-			pages = gpa_node_get_child_from_path (layout, "Pages");
+			pages = gpa_node_get_child_from_path (layout, (const guchar *) "Pages");
 			if (pages) {
 				GPANode *page;
 				gdouble *affines;
@@ -925,7 +898,7 @@ job_parse_config_data (GnomePrintJob *job)
 				while (pagenum < numlp) {
 					guchar *transform;
 					guchar *child_id;
-					child_id = g_strdup_printf ("LP%d", pagenum);
+					child_id = (guchar *) g_strdup_printf ("LP%d", pagenum);
 					page = gpa_node_get_child_from_path (pages, child_id);
 					transform = gpa_node_get_value (page);
 					gpa_node_unref (page);
@@ -940,8 +913,8 @@ job_parse_config_data (GnomePrintJob *job)
 				}
 				gpa_node_unref (pages);
 				if (pagenum == numlp) {
-					pp->num_affines = numlp;
-					pp->affines = affines;
+					job->priv->num_affines = numlp;
+					job->priv->affines = affines;
 				} else {
 					g_free (affines);
 				}
@@ -949,9 +922,9 @@ job_parse_config_data (GnomePrintJob *job)
 		}
 		gpa_node_unref (layout);
 	} else {
-		pp->affines = g_new (gdouble, 6);
-		art_affine_identity (pp->affines);
-		pp->num_affines = 1;
+		job->priv->affines = g_new (gdouble, 6);
+		art_affine_identity (job->priv->affines);
+		job->priv->num_affines = 1;
 	}
 }
 
@@ -961,30 +934,26 @@ job_parse_config_data (GnomePrintJob *job)
 static void
 job_clear_config_data (GnomePrintJob *job)
 {
-	JOBPrivate *pp;
-
 	g_return_if_fail (job->priv);
 
-	pp = job->priv;
-
-	pp->pw = A4_WIDTH;
-	pp->ph = A4_HEIGHT;
-	art_affine_identity (pp->porient);
-	art_affine_identity (pp->lorient);
-	pp->lyw = pp->pw;
-	pp->lyh = pp->ph;
-	pp->num_affines = 0;
-	if (pp->affines) {
-		g_free (pp->affines);
-		pp->affines = NULL;
+	job->priv->pw = A4_WIDTH;
+	job->priv->ph = A4_HEIGHT;
+	art_affine_identity (job->priv->porient);
+	art_affine_identity (job->priv->lorient);
+	job->priv->lyw = job->priv->pw;
+	job->priv->lyh = job->priv->ph;
+	job->priv->num_affines = 0;
+	if (job->priv->affines) {
+		g_free (job->priv->affines);
+		job->priv->affines = NULL;
 	}
-	if (pp->LY_AFFINES) {
-		g_free (pp->LY_AFFINES);
-		pp->LY_AFFINES = NULL;
+	if (job->priv->LY_AFFINES) {
+		g_free (job->priv->LY_AFFINES);
+		job->priv->LY_AFFINES = NULL;
 	}
-	while (pp->LY_LIST) {
-		g_free (pp->LY_LIST->data);
-		pp->LY_LIST = g_list_remove (pp->LY_LIST, pp->LY_LIST->data);
+	while (job->priv->LY_LIST) {
+		g_free (job->priv->LY_LIST->data);
+		job->priv->LY_LIST = g_list_remove (job->priv->LY_LIST, job->priv->LY_LIST->data);
 	}
 }
 
@@ -1132,32 +1101,32 @@ gnome_print_parse_transform (guchar *str, gdouble *transform)
 	art_affine_identity (transform);
 
 	p = str;
-	p = strchr (str, '(');
+	p = (guchar *) strchr ((gchar *) str, '(');
 	if (!p)
 		return FALSE;
 	p += 1;
 	if (!*p)
 		return FALSE;
 	for (i = 0; i < 6; i++) {
-		while (*p && isspace (*p)) p += 1;
-		if (!strncmp (p, "SQRT2", 5)) {
+		while (*p && g_ascii_isspace (*p)) p += 1;
+		if (!strncmp ((gchar *) p, "SQRT2", 5)) {
 			t[i] = M_SQRT2;
-			e = p + 5;
-		} else if (!strncmp (p, "-SQRT2", 6)) {
+			e = (gchar *) p + 5;
+		} else if (!strncmp ((gchar *) p, "-SQRT2", 6)) {
 			t[i] = -M_SQRT2;
-			e = p + 6;
-		} else if (!strncmp (p, "SQRT1_2", 7)) {
+			e = (gchar *) p + 6;
+		} else if (!strncmp ((gchar *) p, "SQRT1_2", 7)) {
 			t[i] = M_SQRT1_2;
-			e = p + 7;
-		} else if (!strncmp (p, "-SQRT1_2", 8)) {
+			e = (gchar *) p + 7;
+		} else if (!strncmp ((gchar *) p, "-SQRT1_2", 8)) {
 			t[i] = -M_SQRT1_2;
-			e = p + 8;
+			e = (gchar *) p + 8;
 		} else {
-			t[i] = g_ascii_strtod (p, &e);
+			t[i] = g_ascii_strtod ((gchar *) p, &e);
 		}
 		if (e == (gchar *) p)
 			return FALSE;
-		p = e;
+		p = (guchar *) e;
 	}
 
 	memcpy (transform, t, 6 * sizeof (gdouble));
@@ -1179,13 +1148,13 @@ gint
 gnome_print_job_print_to_file (GnomePrintJob *job, gchar *output)
 {
 	if (output) {
-		gnome_print_config_set (job->config, "Settings.Transport.Backend",    "file");
-		gnome_print_config_set (job->config, GNOME_PRINT_KEY_OUTPUT_FILENAME, output);
+		gnome_print_config_set (job->config, (const guchar *) "Settings.Transport.Backend",    (const guchar *) "file");
+		gnome_print_config_set (job->config, (const guchar *) GNOME_PRINT_KEY_OUTPUT_FILENAME, (const guchar *) output);
 	} else {
 		/* In the future we might want to use the default or even better,
 		 * go back to the prev. selected printer (Chema)
 		 */
-		gnome_print_config_set (job->config, "Settings.Transport.Backend", "lpr");
+		gnome_print_config_set (job->config, (const guchar *) "Settings.Transport.Backend", (const guchar *) "lpr");
 	}
 	
 	return GNOME_PRINT_OK;
