@@ -41,6 +41,7 @@
 #include "gpa-model.h"
 #include "gpa-printer.h"
 #include "gpa-root.h"
+#include <libgnomeprint/gnome-print-path.h>
 
 typedef struct _GPAPrinterClass GPAPrinterClass;
 struct _GPAPrinterClass {
@@ -203,7 +204,7 @@ gpa_printer_new_from_tree (xmlNodePtr tree)
 	xmlNodePtr node;
 	GPANode *settings, *printer, *model;
 	xmlChar *name, *id, *version;
-	const gchar *lang;
+	const gchar *lang = NULL;
 
 	g_return_val_if_fail (tree != NULL, NULL);
 	g_return_val_if_fail (tree->name != NULL, NULL);
@@ -495,10 +496,10 @@ gpa_printer_list_load (void)
 
 	printers = gpa_list_new (GPA_TYPE_PRINTER, "Printers", TRUE);
 
-	path = g_build_filename (GPA_DATA_DIR, "printers", NULL);
+	path = g_build_filename (gnome_print_data_dir, "printers", NULL);
 	gpa_printer_list_load_from_dir        (printers, path);
 	g_free (path);
-	gpa_printer_list_load_from_module_dir (printers, GPA_MODULES_DIR);
+	gpa_printer_list_load_from_module_dir (printers, gnome_print_modules_dir);
 
 	if (GPA_NODE (printers)->children == NULL) {
 		g_warning ("Could not load any Printer. Check your libgnomeprint installation\n");
