@@ -126,6 +126,8 @@ static const xmlChar *model_unknown_xml_template =
 "        <Option Id=\"NonCollatedCopiesHW\" Type=\"String\" Default=\"true\"/>"
 "        <Option Id=\"CollatedCopiesHW\" Type=\"String\" Default=\"false\"/>"
 "        <Option Id=\"Collate\" Type=\"String\" Default=\"false\"/>"
+"        <Option Id=\"Duplex\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"Tumble\" Type=\"String\" Default=\"false\"/>"
 "        <Option Id=\"PrintToFile\" Type=\"String\" Default=\"false\" Locked=\"true\"/>"
 "        <Option Id=\"FileName\" Type=\"String\" Default=\"output.ps\"/>"
 "      </Option>"
@@ -233,7 +235,7 @@ warn_of_unknown_encoding (char const *encoding)
 }
 
 static char *
-get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
+get_paper_text (ppd_file_t const *ppd, ppd_size_t const *size)
 {
 	/* This is dumb and slow and ugly and crappy and I hate myself. (Dave)*/
 	int i, j, k, len;
@@ -305,7 +307,7 @@ option_list_new_with_default (GPANode      *parent,
 }
 
 static GPANode *
-load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
+load_paper_sizes (ppd_file_t const *ppd, GPANode *parent)
 {
 	ppd_option_t *option;
 	GPANode *node;
@@ -405,8 +407,42 @@ load_cups_hold_types (GPANode *parent)
 	gpa_option_item_new (node,"no-hold",_("immediately"));
 }
 
+static void
+load_cups_duplex (GnomeCupsPrinter *printer, GPANode *job)
+{
+	/* this set is pulled from cups/options.c */
+	char *val = val = gnome_cups_printer_get_option_value (printer, "Duplex");	/* std */
+	if (NULL == val)
+		val = gnome_cups_printer_get_option_value (printer, "JCLDuplex");	/* Samsung */
+	if (NULL == val)
+		val = gnome_cups_printer_get_option_value (printer, "EFDuplex");	/* EFI */
+	if (NULL == val)
+		val = gnome_cups_printer_get_option_value (printer, "KD03Duplex");	/* Kodak */
+	if (NULL != val) {
+		gboolean duplex = FALSE, bind_short_edge = FALSE;
+
+		/* we do not really need to ignore case, but it does not hurt */
+		if (g_ascii_strcasecmp (val, "None") == 0) {
+			/* this is the default */
+		} else if (g_ascii_strcasecmp (val, "DuplexNoTumble") == 0) {
+			duplex = TRUE;
+			bind_short_edge = FALSE;
+		} else if (g_ascii_strcasecmp (val, "DuplexTumble") == 0) {
+			duplex = TRUE;
+			bind_short_edge = TRUE;
+		} else {
+			g_warning ("Unknown Duplex setting == '%s'", val);
+		}
+		g_free (val);
+		gpa_option_string_new (job, "Duplex",
+			duplex ? "true" : "false");
+		gpa_option_string_new (job, "Tumble",
+			(duplex && bind_short_edge) ? "true" : "false"); /* be anal */
+	}
+}
+
 static GPAModel *
-get_model (const gchar *printer, ppd_file_t *ppd)
+get_model (GnomeCupsPrinter *cupsprinter, ppd_file_t *ppd)
 {
 	GPANode *media;
 	GPANode *model;
@@ -435,6 +471,7 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	load_paper_sizes   (ppd, media);
 	load_paper_sources (ppd, output);
 	load_cups_hold_types (job);
+	load_cups_duplex (cupsprinter, job);
 
 	gpa_node_unref (output);
 	gpa_node_unref (media);
@@ -478,46 +515,13 @@ static void
 gnome_print_cups_adjust_settings (GPANode	   *settings,
 				  GnomeCupsPrinter *printer)
 {
-	/* default to single sided */
-	char *val;
-
-	val = gnome_cups_printer_get_option_value (printer, "PageSize");
+	char *val = gnome_cups_printer_get_option_value (printer, "PageSize");
 	if (val != NULL) {
 		gpa_node_set_path_value (settings,
 			"Output.Media.PhysicalSize", val);
 		g_free (val);
 	}
 
-	/* this set is pulled from cups/options.c */
-	val = gnome_cups_printer_get_option_value (printer, "Duplex");	/* std */
-	if (NULL == val)
-		val = gnome_cups_printer_get_option_value (printer, "JCLDuplex");	/* Samsung */
-	if (NULL == val)
-		val = gnome_cups_printer_get_option_value (printer, "EFDuplex");	/* EFI */
-	if (NULL == val)
-		val = gnome_cups_printer_get_option_value (printer, "KD03Duplex");	/* Kodak */
-	if (NULL != val) {
-		GPANode *job = gpa_node_lookup (settings, "Output.Job");
-		gboolean duplex = FALSE, bind_short_edge = FALSE;
-
-		/* we do not really need to ignore case, but it does not hurt */
-		if (g_ascii_strcasecmp (val, "None") == 0) {
-			/* this is the default */
-		} else if (g_ascii_strcasecmp (val, "DuplexNoTumble") == 0) {
-			duplex = TRUE;
-			bind_short_edge = FALSE;
-		} else if (g_ascii_strcasecmp (val, "DuplexTumble") == 0) {
-			duplex = TRUE;
-			bind_short_edge = TRUE;
-		} else {
-			g_warning ("Unknown Duplex setting == '%s'", val);
-		}
-		g_free (val);
-		gpa_option_string_new (job, "Duplex",
-			duplex ? "true" : "false");
-		gpa_option_string_new (job, "Tumble",
-			(duplex && bind_short_edge) ? "true" : "false"); /* be anal */
-	}
 }
 
 static void
@@ -583,13 +587,12 @@ cb_get_ppd (guint id, ppd_file_t *ppd, GError **error, gpointer user_data)
 	printers_list = data->list;
 	name = gnome_cups_printer_get_name (data->printer);
 
-	if (ppd) {
-		model = get_model (name, ppd);
-	} else {
+	if (NULL == ppd) {
 		g_message ("The ppd file for the CUPS printer %s "
 			   "could not be loaded.", name);
 		model = get_model_no_ppd (name);
-	}
+	} else
+		model = get_model (cupsprinter, ppd);
 
 	if (model == NULL) {
 		g_warning ("Couldn't create model for %s!\n", name);
@@ -612,9 +615,9 @@ cb_get_ppd (guint id, ppd_file_t *ppd, GError **error, gpointer user_data)
 						     GPA_SETTINGS (settings));
 
 		if (success) {
-			gpa_list_prepend (printers_list, printer);
 			if (gnome_cups_printer_get_is_default (cupsprinter))
 				gpa_list_set_default (printers_list, printer);
+			gpa_list_prepend (printers_list, printer);
 		}
 	} else {
 		g_warning ("The CUPS printer %s could not be created\n", name);
