@@ -33,10 +33,10 @@
  *
  */
 
-/* $Id: list.c,v 1.2 2004/07/21 16:42:25 jody Exp $ */
+/* $Id: list.c,v 1.4 2004/09/10 18:59:06 jody Exp $ */
 /* @(#)list.c 1.7 03/02/06 SMI */
 
-/**
+/*
  * @file list.c
  * @brief Bidirectional list class
  * @author Alexander Gelfenbain
@@ -89,7 +89,7 @@ static lnode *appendPrim(list this, void *el)
         blink = this->tail;
     } else {
         flink = &this->head;
-        blink = 0;
+        blink = NULL;
         this->cptr = ptr;                         /*- list was empty - set current to this element */
     }
 
@@ -97,7 +97,7 @@ static lnode *appendPrim(list this, void *el)
     this->tail = ptr;
 
     ptr->prev = blink;
-    ptr->next = 0;
+    ptr->next = NULL;
 
     this->aCount++;
     return ptr;
@@ -113,7 +113,7 @@ static lnode *prependPrim(list this, void *el)
         flink = this->head;
     } else {
         blink = &this->tail;
-        flink = 0;
+        flink = NULL;
         this->cptr  = ptr;                        /*- list was empty - set current to this element */
     }
 
@@ -121,7 +121,7 @@ static lnode *prependPrim(list this, void *el)
     this->head   = ptr;
 
     ptr->next = flink;
-    ptr->prev = 0;
+    ptr->prev = NULL;
 
     this->aCount++;
     return ptr;
@@ -135,8 +135,8 @@ list listNewEmpty(void)                           /*- default ctor */
     assert(this != 0);
 
     this->aCount = 0;
-    this->eDtor = 0;
-    this->head = this->tail = this->cptr = 0;
+    this->eDtor = NULL;
+    this->head = this->tail = this->cptr = NULL;
 
     return this;
 }
@@ -153,8 +153,8 @@ list listNewCopy(list l)                          /*- copy ctor */
     ptr = l->head;
 
     this->aCount = 0;
-    this->eDtor = 0;
-    this->head = this->tail = this->cptr = 0;
+    this->eDtor = NULL;
+    this->head = this->tail = this->cptr = NULL;
 
     while (ptr) {
         c = appendPrim(this, ptr->value);
@@ -178,8 +178,8 @@ list listNewConcat(list lhs, list rhs)
 
 
     this->aCount = 0;
-    this->eDtor = 0;
-    this->head = this->tail = this->cptr = 0;
+    this->eDtor = NULL;
+    this->head = this->tail = this->cptr = NULL;
 
     ptr = lhs->head;
     while (ptr) {
@@ -462,7 +462,7 @@ list   listInsertBefore(list this, void *el)
 
 list   listRemove(list this)
 {
-    lnode *ptr = 0;
+    lnode *ptr = NULL;
     if (this->cptr == 0) return this;
 
     if (this->cptr->next != 0) {
@@ -499,7 +499,7 @@ list   listClear(list this)
         node = ptr;
     }
 
-    this->head = this->tail = this->cptr = 0;
+    this->head = this->tail = this->cptr = NULL;
     assert(this->aCount == 0);
     return this;
 }
