@@ -9,9 +9,6 @@
  * Authors: Chema Celoiro <chema@ximian.com>
  */
 
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/mman.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <popt.h>
@@ -19,7 +16,12 @@
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <ctype.h>
+#include <unistd.h>
+#include <sys/types.h>
+
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif
 
 static gint debug;
 
@@ -502,7 +504,12 @@ my_read_file (gchar *filename, gint *size)
 		my_error ("Could not stat %s", filename);
 	}
 	*size = s.st_size;
+#ifndef HAVE_MMAP
+	my_error ("mmap unsupported on this platform");
+	buf = NULL;
+#else
 	buf = mmap (NULL, s.st_size, PROT_READ, MAP_SHARED, fh, 0);
+#endif
 	close (fh);
 	if ((buf == NULL) || (buf == (void *) -1)) {
 		my_error ("Buff is NULL after mmaping");
